// WposCtrl.cpp : implementation file
//
//

#include "stdafx.h"
#include "WposCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MIN_POS 0						//control range
#define MAX_POS 10000

#define SEGMENT_COLOR RGB(128,255,255)		//light cyan threshold bar color
#define MKR_COLOR RGB(255,255,0)		//light yellow marker color
#define BACK_COLOR RGB(0,0,128)			// background
/////////////////////////////////////////////////////////////////////////////
// CWposCtrl

CWposCtrl::CWposCtrl()
{
    RegisterWindowClass();
	m_SegStartPos = 0;
	m_SegStopPos = 0;
	m_LeftMkrPos = 0;
	m_RightMkrPos = 0;

	m_pParentWnd = NULL;
	m_pCallback = NULL;
	m_TotalSecs = MAX_POS/2;
}

CWposCtrl::~CWposCtrl()
{
}

// Register the window class if it has not already been registered.
BOOL CWposCtrl::RegisterWindowClass()
{
    WNDCLASS wndcls;
    HINSTANCE hInst = AfxGetInstanceHandle();

    if (!(::GetClassInfo(hInst, CWPOSCTRL_CLASSNAME, &wndcls)))
    {
        // otherwise we need to register a new class
        wndcls.style            = CS_HREDRAW | CS_VREDRAW|CS_DBLCLKS;
        wndcls.lpfnWndProc      = ::DefWindowProc;
        wndcls.cbClsExtra       = wndcls.cbWndExtra = 0;
        wndcls.hInstance        = hInst;
        wndcls.hIcon            = NULL;
        wndcls.hCursor          = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
        wndcls.hbrBackground    = NULL;	//am drawing own background color
        wndcls.lpszMenuName     = NULL;
        wndcls.lpszClassName    = CWPOSCTRL_CLASSNAME;

        if (!AfxRegisterClass(&wndcls))
        {
            AfxThrowResourceException();
            return FALSE;
        }
    }

    return TRUE;
}

BEGIN_MESSAGE_MAP(CWposCtrl, CWnd)
	//{{AFX_MSG_MAP(CWposCtrl)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////////////////////
//  Creates the window
///////////////////////////////////////////////////////////////////////////
BOOL CWposCtrl::Create( DWORD dwStyle, const RECT& rect,
							CWnd* pParentWnd, UINT nID)
{
	return CWnd::Create(CWPOSCTRL_CLASSNAME, _T(""), dwStyle, rect, pParentWnd, nID);

}

void CWposCtrl::PreSubclassWindow() 
{
	ModifyStyleEx (0, WS_EX_CLIENTEDGE|WS_EX_NOPARENTNOTIFY);
	ModifyStyle (0, WS_BORDER|WS_CHILD|WS_VISIBLE, SWP_DRAWFRAME);
//	ModifyStyle (0, WS_DLGFRAME|WS_CHILD|WS_VISIBLE, SWP_DRAWFRAME);
	m_pParentWnd = NULL;
	m_pCallback = NULL;
	
	CWnd::PreSubclassWindow();
}

////////////////////////////////////////////////////////////////////////////
//  Set Segment Start Seconds
///////////////////////////////////////////////////////////////////////////
void CWposCtrl::SetSegStartSecs(INT secs)
{
CRect r;
	m_SegStartSecs = secs;
	GetClientRect(r);
	InvalidateRect(r);
}

////////////////////////////////////////////////////////////////////////////
//  Set Segment Stop Seconds
///////////////////////////////////////////////////////////////////////////
void CWposCtrl::SetSegStopSecs(INT secs)
{
CRect r;
	m_SegStopSecs = secs;
	GetClientRect(r);
	InvalidateRect(r);
}

////////////////////////////////////////////////////////////////////////////
//  Set Left Marker Seconds
///////////////////////////////////////////////////////////////////////////
void CWposCtrl::SetLMkrSecs(INT secs)
{
CRect r;
	m_LeftMkrSecs = secs;
	GetClientRect(r);
	InvalidateRect(r);
}

////////////////////////////////////////////////////////////////////////////
//  Set Right Marker Seconds
///////////////////////////////////////////////////////////////////////////
void CWposCtrl::SetRMkrSecs(INT secs)
{
CRect r;
	m_RightMkrSecs = secs;
	GetClientRect(r);
	InvalidateRect(r);
}


/////////////////////////////////////////////////////////////////////////////
// CWposCtrl message handlers
//
//       Called to draw the control
/////////////////////////////////////////////////////////////////////////////
void CWposCtrl::OnPaint() 
{
CDC MemDC;
CPaintDC dc(this); // device context for painting
//create various rectangles for all the viewable things
	GetClientRect( m_rClient );	

	if( m_bmImage.m_hObject != NULL )	// delete old bitmap object
		m_bmImage.DeleteObject();
//create a new bitmap with current screen DC settings
	m_bmImage.CreateCompatibleBitmap(&dc,m_rClient.Width(),m_rClient.Height() );
//create memory DC pointer to the image bitmap
	MemDC.CreateCompatibleDC( &dc);
	CBitmap* pbmOld = MemDC.SelectObject( &m_bmImage );

//Fill controlbar with background color
	MemDC.FillSolidRect(m_rClient,BACK_COLOR);

	m_LeftMkrPos = (m_rClient.Width()*m_LeftMkrSecs)/(m_TotalSecs);
	m_RightMkrPos = (m_rClient.Width()*m_RightMkrSecs)/(m_TotalSecs);
	m_SegStartPos = (m_rClient.Width()*m_SegStartSecs)/(m_TotalSecs);
	m_SegStopPos = (m_rClient.Width()*m_SegStopSecs)/(m_TotalSecs);

//Draw rectangle for the segment bar depending upon m_SegStartPos. 	
	CRect rSeg = m_rClient;
	rSeg.left = m_SegStartPos;
	if(m_SegStartPos<m_SegStopPos)
		rSeg.right = m_SegStopPos;
	else
		rSeg.right = rSeg.left+1;
//Fill segment bar with segment color
	MemDC.FillSolidRect(rSeg,SEGMENT_COLOR);

//Draw Left and right marker rectangle
	rSeg.top = (3*m_rClient.Height())/8;
	rSeg.bottom = (5*m_rClient.Height())/8;
	rSeg.left = m_LeftMkrPos;
	rSeg.right = m_RightMkrPos;
	MemDC.FillSolidRect(rSeg,MKR_COLOR);
	

// Copy bitmap into screen context
	dc.BitBlt(	m_rClient.left, m_rClient.top,		//dest left,top
				m_rClient.right, m_rClient.bottom,	//dest width, Height
				&MemDC,							//src dc
				0,0,							//src left, top
				SRCCOPY);
//deselect bitmap
	MemDC.SelectObject( pbmOld );	// from DC
}


/////////////////////////////////////////////////////////////////////////////
// change threshold when mouse button is clicked in the control
/////////////////////////////////////////////////////////////////////////////
void CWposCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{
CRect r;
INT secs;
	GetClientRect( r );	
	secs = (point.x*(m_TotalSecs))/r.Width();
	if(nFlags&MK_CONTROL)
	{
		if(secs<m_RightMkrSecs)
		{
			if( m_pParentWnd && m_pCallback)
				(m_pParentWnd->*m_pCallback)( MSG_TYPE_LMKR, secs );
		}
	}
	else
	{
		if( m_pParentWnd && m_pCallback)
			(m_pParentWnd->*m_pCallback)( MSG_TYPE_SEGMENT, secs );	
	}
	CWnd::OnLButtonDown(nFlags, point);
}



void CWposCtrl::OnRButtonDown(UINT nFlags, CPoint point) 
{
CRect r;
INT secs;
	if(nFlags&MK_CONTROL)
	{
		GetClientRect( r );	
		secs = (point.x*(m_TotalSecs))/r.Width();
		if(secs>m_LeftMkrSecs)
		{
			if( m_pParentWnd && m_pCallback)
				(m_pParentWnd->*m_pCallback)( MSG_TYPE_RMKR, secs );
		}
	}
	CWnd::OnRButtonDown(nFlags, point);
}
